#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <unistd.h>

#include "../mc-utils.h"
#include <libprocess/gwyprocess.h>
#include <libgwyddion/gwymath.h>

extern void odr_wrap_implicit_(int *n, int *m, int *np, int *nq, double *x, double *y, double *beta, 
		double *ux, double *uy, double *rho, double *stdev_beta, double *cov, double *res_var, double *res_sun, int *info );

typedef struct {
	int ndata;
	double *hc;
	double *Ac;
	double *uhc;
	double *uAc;
	double *rho;  /* correlation between h_i - A_i */
	double **corr; /* correlation between A_i - A_j */
} AreaData;	

typedef struct {
	int ndata;
	double *S;
	double *uS;
	double *covSh;
} SData;

typedef struct {
	int ndata;
	double *F;
	double *uF;
} FData;

typedef struct {
	double Ercalc, uErcalcS, uErcalcAhc, uErcalcApar, uErcalccov, uErcalc;
} ErData;

typedef struct {
	double A, uAhc, uApar, uA;
} AData;

typedef struct {
	double H, uHAhc, uHApar, uH, uF;
} HData;






void print_help()
{
	printf(" fit the area calibration file with an Oliver Pharr Polynomial\n");
	printf(" -h    show this help\n");
	printf(" -f    input file \n");
	printf(" -p    degree of Oliver Pharr polynomial (default is 5) \n");
	printf(" -n    don't use correlation (default is use correlation) \n");
	printf(" -m    number of Monte Carlo iterations (default = 1) \n");
	printf(" -i    print individual random area curves \n");
	printf(" -c    file containing correlation between area-data \n");
}

void free_area_data(AreaData *data){
	int i;

	free(data->hc);
	free(data->Ac);
	free(data->uhc);
	free(data->uAc);
	free(data->rho);
	if (data->corr){
		for (i = 0 ; i < data->ndata; i++) 
			free(data->corr[i]);
		free(data->corr);
	}
}

void free_s_data(SData *data){
	free(data->S);
	free(data->uS);
	free(data->covSh);
}

void free_f_data(FData *data){
	free(data->F);
	free(data->uF);
}


int read_corr_data( char *fnm, AreaData *data)
{
	FILE *file;
	int i, j, ndata, n;
	double x;
	size_t size; 
	char *line = NULL;

	file = fopen(fnm, "r");

	if (!file)
		return -1;

	ndata = 0;
	while (getline(&line, &size, file) != EOF) {
		ndata ++;
	}

	printf(" corr ndata %d \n", ndata);
	if (ndata != data -> ndata ){
		printf("read only %d data instead of %d \n", ndata, data->ndata);
		return -1;
	}

	rewind(file);

	data->corr = (double **)malloc(ndata * sizeof(double*)); 
	for (i = 0; i< ndata; i++){
		data->corr[i] = (double *)malloc(ndata *sizeof(double ));
	}
		
	for (i = 0; i< ndata; i++){
		data->corr[i][i] = 0;
		for (j = 0; j< i; j++){
			n = fscanf(file, " %lf ", &x);
			if (!n)
				return -1;
			data->corr[i][j] = x;
			data->corr[j][i] = x;
		}
	}

	return ndata;

}

int read_area_data( char *fnm, AreaData *data, SData *sdata, FData *fdata)
{
	FILE *file;
	int ndata = 0;
	int i;
	size_t size; 
	char *line = NULL;
	double x1, x2, x3, x4, x5, x6, x7, x8, x9, x10, x11;

	file = fopen(fnm, "r");

	if (!file)
		return -1;

	while (getline(&line, &size, file) != EOF) {
		if (strstr(line, "#") == NULL)
			ndata ++;
	}
	printf("ndata %d \n", ndata);

	data->ndata = ndata;
	data->hc = (double *)malloc(ndata *sizeof(double));
	data->Ac = (double *)malloc(ndata *sizeof(double));
	data->uhc = (double *)malloc(ndata *sizeof(double));
	data->uAc = (double *)malloc(ndata *sizeof(double));
	data->rho = (double *)malloc(ndata *sizeof(double));

	sdata->ndata = ndata;
	sdata->S = (double *)malloc(ndata *sizeof(double));
	sdata->uS = (double *)malloc(ndata *sizeof(double));
	sdata->covSh = (double *)malloc(ndata *sizeof(double));

	fdata->ndata = ndata;
	fdata->F = (double *)malloc(ndata *sizeof(double));
	fdata->uF = (double *)malloc(ndata *sizeof(double));

	rewind(file);

	ndata = 0; 
	while (getline(&line, &size, file) != EOF) {
		if (strstr(line, "#") == NULL){
			i = sscanf(line, "%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf", &x1, &x2, &x3, &x4, &x5, &x6, &x7, &x8, &x9, &x10, &x11);
			if (i == 11 ){
				data->hc[ndata] = x1;
				data->Ac[ndata] = x2*1e6;
				data->uhc[ndata] = x3;
				data->uAc[ndata] = x4*1e6;
				data->rho[ndata] = x6;
				sdata->S[ndata] = x7;
				sdata->uS[ndata] = x8;
				sdata->covSh[ndata] = x9;
				fdata->F[ndata] = x10;
				fdata->uF[ndata] = x11;
				ndata++;
			}
		}
	}
	printf("ndata %d data->ndata %d \n", ndata, data->ndata);
	if (ndata != data -> ndata ){
		printf("read only %d data instead of %d \n", ndata, data->ndata);
		return ndata;
	}

	return ndata;
}

void	set_corr_zero(AreaData *data){
	int i;

	for (i = 0; i < data->ndata; i++){
		data->rho[i] = 0;
	}
}

int assemble_amat(AreaData *data,  double *amat)
{
	int i, j, k;
	int n;
	double **cov;
	double *p;

	n = data->ndata;
	cov = (double **)malloc(2*n*sizeof(double *));
	for (i = 0; i < 2*n; i++){
		cov[i] = (double *)malloc(2*n*sizeof(double));
	}
	for (i = 0; i < 2*n; i++){
		for (j = 0; j < 2*n; j++){
			cov[i][j] = 0;
		}
	}

	for (i = 0; i < n; i++){
		cov[i][i] = data->uhc[i]*data->uhc[i];
	}
	k = 0;
	for (i = 0; i < n; i++){
			cov[i][i+n] = data->rho[k]*data->uhc[i]*data->uAc[i];
			cov[i+n][i] =cov[i][i+n] ;
			k++;
	}
	for (i = 0; i < n; i++){
		cov[i+n][i+n] = data->uAc[i]*data->uAc[i];

	}
	for (i = 0; i < n; i++){
		for (j = 0; j < i; j++){
			cov[i+n][j+n] = data->corr[i][j]*data->uAc[i]*data->uAc[j];
			cov[j+n][i+n] = cov[i+n][j+n];
		}
	}
	
	/*
	for (i = 0; i < 2*n; i++){
		for (j = 0; j < 2*n; j++){
			printf(" %g ", cov[i][j]);
		}
		printf("\n");
	}
	
	*/
	p = amat;
	for (i = 0; i < 2*n; i++){
		for (j = 0; j <= i; j++, p++){
			*p = cov[i][j];
		}
	}

	p = amat;
	/*
	printf("amat: \n");
	for (i = 0; i < n*(2*n+1); i++, p++){
		printf("%g ", *p);
	}
	*/

	if (! gwy_math_choleski_decompose(2*n, amat)){
                fprintf(stderr, "unable to decompose covariance matrix\n");
		return 0;
        }

	/*
	p = amat;
	printf("\n decomposed amat: \n");
	for (i = 0; i < n*(2*n+1); i++, p++){
		printf("%g ", *p);
	}
	*/
	

	for (i = 0; i < n; i++){
		free(cov[i]);
	}
	free(cov);
	return 2*n;
}

void calc_noise_inter(int n, double *xnoise, double *amat, GRand *rng)
{
	gdouble *p, *z;
	int i, j;

	for (i = 0; i< 2*n; i++){
		xnoise[i] = 0;
	}

	z = (gdouble *)malloc(2*n*sizeof(gdouble));
	for (i = 0; i< 2*n; i++){
		z[i] = gaussian_random_number(rng);
	}

	p = amat;
	for (i = 0; i< 2*n; i++){
		for (j = 0; j<= i; j++, p++){
			xnoise[i] += (*p)*z[j];
/*			if (*p*z[j] > 1e10)
				printf("i %d j %d %g \n", i, j, *p);
				*/
		}
		/*
		if (xnoise[i] > 1e10)
			printf("i %d %g \n",i, xnoise[i]);
			*/
	}
}

double calc_A(double x, double *coef, int np)
{
	int i;
	double A = 0;


	A = 24.5*x*x;
	for ( i = 0; i< np; i++){
		A+= coef[i]*pow(x, pow(2, -i ));
	}	
	return A;
}

double calc_uA_covmatrix(double x, double *coef, double **cov_coef ,  int np)
{
	int i,j;
	double uA = 0;
	double *dAdp; 

	dAdp = (double *)malloc(np*sizeof(double));

	for ( i = 0; i< np; i++){
		dAdp[i] = pow(x, pow(2, -i));
	}


	/*
	   for ( i = 0; i< np; i++){
	   uA+= dAdp[i]*dAdp[i]*unc_coef[i]*unc_coef[i];
	   }
	   k =0;
	   for ( i = 0; i< np; i++){
	   for ( j = i+1; j< np; j++){
	   uA += 2*cov[k++] *dAdp[i]*dAdp[j]*unc_coef[i]*unc_coef[j];	
	   }
	   }
	 */
	uA = 0;
	for ( i = 0; i< np; i++){
		for ( j = 0; j< np; j++){
		//	printf(" %d %d %g \n", i, j, cov_coef[i][j]);
			uA += cov_coef[i][j]*dAdp[i]*dAdp[j];
			//printf("uA %g \n",uA);
		}
	}
	free (dAdp);

	return sqrt(uA);
}

double calc_Ac_der(double x, double *coef, int np)
{
	int i;
	double dAdh = 0; 

	dAdh = 24.5*2*x;
	for ( i = 0; i< np; i++){
		dAdh+= coef[i]*pow(2, -i )*pow(x, pow(2, -i )-1);
	}	
	return dAdh;
}

void  calc_unc_analysis(AreaData *data, SData *sdata, FData *fdata, int i, double *coef, double  **cov, int np, ErData *Erdata, AData *Adata, HData *Hdata)
{
	double hc, uhc, S, uS, covSh;
	double dAdh;
	double A, uApar, uAhc, uA;
	double Ercalc, uErcalcS, uErcalcAhc, uErcalcApar, uErcalccov, uErcalc;
	double F, uF, H, uHAhc, uHApar, uHF, uH;

	hc = data->hc[i];
	uhc = data->uhc[i];
	S = sdata->S[i];
	uS = sdata->uS[i];
	covSh = sdata->covSh[i];
	F = fdata->F[i];
	uF = fdata->uF[i];

	A = calc_A(hc, coef, np);
	uApar = calc_uA_covmatrix(hc, coef, cov, np);
	dAdh = calc_Ac_der(hc, coef, np);
	uAhc = fabs(dAdh*uhc);
	uA = sqrt(uAhc*uAhc + uApar*uApar);

	/*conversion factors */
	Ercalc = sqrt(M_PI/A)*S/2.*1e6;

	uErcalcS = Ercalc/S*uS;
	uErcalcAhc = 1./2*Ercalc/A*uAhc;
	uErcalcApar = 1./2*Ercalc/A*uApar;
	uErcalccov = 2*Ercalc/S*1./2*Ercalc/A*dAdh*covSh;

	uErcalc = sqrt( uErcalcS*uErcalcS + uErcalcAhc*uErcalcAhc + uErcalcApar*uErcalcApar + uErcalccov);


	H = F/A*1e9; 
	uHAhc = H/A *uAhc;
	uHApar= H/A *uApar;
	uHF = H/F*uF;

	uH = sqrt( uHAhc*uHAhc + uHApar*uHApar + uHF*uHF);

	Adata->A = A;
	Adata->uAhc = uAhc;
	Adata->uApar = uApar;
	Adata->uA = uA;


	Erdata-> Ercalc = Ercalc;
	Erdata-> uErcalcS = uErcalcS;
	Erdata-> uErcalcAhc = uErcalcAhc;
	Erdata-> uErcalcApar = uErcalcApar;
	Erdata-> uErcalccov = uErcalccov;
	Erdata-> uErcalc = uErcalc;

	Hdata->H = H;
	Hdata->uHAhc = uHAhc;
	Hdata->uHApar = uHApar;
	Hdata->uF = uF;
	Hdata->uH = uH;
}

int main (int argc, char *argv[])
{

	int c, i, j;
	char *fnm = NULL;
	char *corrfnm = NULL;
	AreaData data;
	SData sdata;
	FData fdata;
	ErData Erdata;
	AData Adata;
	HData Hdata;
	int n;
	/*	double SSres, sxx, syy, R2, R2adj;
		double xavg, yavg ; */
	int usecorr = 1;
	double *beta;
	double *stdev_beta;
	double *cov;
	int np = 5;
	int m, nq;
	double *x, *y;
	double *ux, *uy, *rho;
	double *amat;
	double res_var, res_sum;
	double *xnoise;
	int info;
	double *mcresvar, *mcressum, **mcbeta, **mcstd;
	char str[400];
	int Nmc = 1, nstat;
	gdouble *xhist, *yhist;
	gdouble *mcavg, **mccov;
	int print_xy = 0;
	double to = 500;
	int num = 500;
	double dx, AA, uAA;
	FILE *A, *B, *C;
	GRand *rng;


	if (argc == 1 ){
		print_help();
		exit(0);
	}

	data.corr = NULL;
	while (( c = getopt (argc , argv, "c:f:hip:nm:")) != -1 ){
		switch (c)
		{
			case 'f':
				fnm = optarg;
				break;
			case 'p':
				np = atoi(optarg);
				break;
			case 'n':
				usecorr = 0;
				break;
			case 'i':
				print_xy = 1;
				break;
			case 'm':
				Nmc = atoi(optarg);
				break;
			case 'c':
				corrfnm = optarg;
				break;
			case 'h':
			default:
				print_help();
				exit(0);
				break;
		}
	}


	if (np > 10){
		printf(" WARNING: polynomial degree more than 10 not defined. \n");
		printf(" If you really want such a high degree of the polynom, modify the code and recompile. \n");
		exit(1);
	}
	n = read_area_data( fnm, &data, &sdata, &fdata);
	if (n == -1){
		printf(" Failed to load data from file %s\n", fnm);
		exit(1);
	}
	if (!usecorr)
		set_corr_zero(&data);

	if (corrfnm){
		n = read_corr_data(corrfnm, &data);
	}
	if (n == -1){
		printf(" Failed to load data from file %s\n", corrfnm);
		exit(1);
	}
	/*
	for (i = 0 ; i< n; i++){
		for (j = 0 ; j< n; j++){
		printf(" %g ", data.corr[i][j]);
		}
		printf(" \n");
	}
	*/

	/* set up stuff for ODRPACK */
	m = 2;
	nq = 1;
	n = data.ndata;

	beta = (double *) calloc(np,sizeof(double));
	stdev_beta = (double *) calloc(np,sizeof(double));
	cov = (double *) calloc(np*(np-1)/2, sizeof(double));

	x = (double *)malloc(m*n *sizeof(double));
	xnoise = (double *)malloc(m*n * sizeof(double));
	ux = (double *)malloc(m*n *sizeof(double));
	y = (double *)malloc(m*n *sizeof(double));
	uy = (double *)malloc(m*n *sizeof(double));

	for (i = 0; i< n; i++){
		ux[i] = data.uhc[i];
	}
	for (i = 0; i< n; i++){
		ux[n + i] = data.uAc[i];
	}
	rho = data.rho;

	mcbeta = (double **) malloc(np*sizeof(double *));
	mcstd = (double **) malloc(np*sizeof(double *));

	for (i = 0; i < np; i++){
		mcbeta[i] = (double *) malloc(Nmc*sizeof(double));
		mcstd[i] = (double *) malloc(Nmc*sizeof(double));
	}
	mcresvar = (double *) malloc(Nmc*sizeof(double ));
	mcressum = (double *) malloc(Nmc*sizeof(double ));

	amat = (double *)malloc(n*(2*n+1)*sizeof(double));

	if (!assemble_amat(&data, amat))
		exit(0);

	rng= g_rand_new();
	for (j = 0; j < Nmc; j++){
		calc_noise_inter(n,  xnoise, amat, rng);

		for (i = 0; i< n; i++){
			x[i] = data.hc[i] + xnoise[i];
		}
		for (i = 0; i< n; i++){
			x[n + i] = data.Ac[i] + xnoise[n +i];
		}

		odr_wrap_implicit_( &n, &m, &np, &nq, x, y, beta, ux, uy, rho, stdev_beta, cov, &res_var, &res_sum, &info);
		for (i = 0; i< np; i++){
			mcbeta[i][j] = beta[i];
			mcstd[i][j] = stdev_beta[i]; 
		}
		mcresvar[j] = res_var;
		mcressum[j] = res_sum;


		if (print_xy){
			snprintf(str, 200, "res_%d", j);
			A = fopen(str, "w");
			for (i = 0; i< n; i++)
				fprintf(A, "%g %g %g %g \n", x[i], x[n +i],  x[i], x[n +i]);

			fprintf(A,"\n\n #");
			for (i = 0; i < np; i++){
				fprintf(A," %g ", beta[i]);
			}
			fprintf(A, "\n#");
			for (i = 0; i < np; i++){
				fprintf(A," %g ", stdev_beta[i]);
			}
			fprintf(A,"\n#");
			fprintf(A," %d \n#", info);
			fprintf(A," %g %g \n", res_var, res_sum);

			fclose(A);
		}
		/*
		   for (i = 0; i < np; i++){
		   printf(" beta[%d]:   %g    +-  %g   (%g %%) \n", i, beta[i], stdev_beta[i], stdev_beta[i]/beta[i]*100);
		   }
		 */
	}


	if (Nmc >= 10){
		nstat = (gint)floor(3.49*cbrt(Nmc)+0.5);
		xhist = (double *) malloc(nstat*sizeof(double));
		yhist = (double *) malloc(nstat*sizeof(double));
		printf("nstat %d \n", nstat);

		if (!usecorr)
			snprintf(str, 200, "histograms_nocorr_inter_%d_%s_np%d", Nmc, fnm, np);
		else
			snprintf(str, 200, "histograms_inter_%d_%s_np%d", Nmc, fnm, np);

		A = fopen(str, "w");
		for (j = 0; j< np; j++){
			create_histogram(mcbeta[j], Nmc, xhist, yhist, nstat);
			for (i =0; i< nstat; i++)
				fprintf(A, "%g %g \n", xhist[i], yhist[i]);
			fprintf(A, "\n\n");
		}

		for (j = 0; j< np; j++){
			create_histogram(mcstd[j], Nmc, xhist, yhist, nstat);
			for (i =0; i< nstat; i++)
				fprintf(A, "%g %g \n", xhist[i], yhist[i]);
			fprintf(A, "\n\n");
		}

		create_histogram(mcresvar, Nmc, xhist, yhist, nstat);
		for (i =0; i< nstat; i++)
			fprintf(A, "%g %g \n", xhist[i], yhist[i]);
		fprintf(A, "\n\n");

		create_histogram(mcressum, Nmc, xhist, yhist, nstat);
		for (i =0; i< nstat; i++)
			fprintf(A, "%g %g \n", xhist[i], yhist[i]);
		fprintf(A, "\n\n");


		fclose(A);
	}

	printf("stats done \n");
	

	if (!usecorr)
		snprintf(str, 200, "stats_nocorr_inter_%d_%s_np%d", Nmc, fnm, np);
	else
		snprintf(str, 200, "stats_inter_%d_%s_np%d", Nmc, fnm, np);

	B = fopen(str, "w"); 
	mcavg = (double *)malloc(np* sizeof(double));
	mccov = (double **)malloc(np * sizeof(double *));
	for (i = 0; i< np; i++){
		mccov[i] = (double *) malloc(np* sizeof(double));
	}

	for (i = 0; i< np; i++){
		mcavg[i] = 0;
		for (j = 0; j< np; j++){
			mccov[i][j] = 0;
		}
	}

	
	fprintf(B,"beta: \n");
	printf("beta: \n");
	for (i =0; i< np; i++){
		for (m= 0; m< Nmc; m++){
			mcavg[i] += mcbeta[i][m];
		}
		mcavg[i] /= Nmc;
		fprintf(B,"%g ", mcavg[i]);
		printf("%g ", mcavg[i]);
	}
	fprintf(B, "\n");
	fprintf(B, "\ncovariance: \n");
	printf( "\n");
	printf( "\ncovariance: \n");
	for (i =0; i< np; i++){
		for (j =i; j< np; j++){
			for (m= 0; m< Nmc; m++){
				mccov[i][j] += (mcbeta[i][m] - mcavg[i])*(mcbeta[j][m]-mcavg[j]);

			}
			mccov[i][j] /= Nmc;
		}
		for (j =0; j< i; j++)
			mccov[i][j] = mccov[j][i];
	}
	for (i =0; i< np; i++){
		for (j =0; j< np; j++){
			fprintf(B, " %g ", mccov[i][j]);
			printf( " %g ", mccov[i][j]);
		}
		fprintf(B, "\n");
		printf( "\n");
	}


	fclose(B);

	if ( !usecorr)
		snprintf(str, 200, "area_%d_%s_np%d_nocorr_inter", Nmc, fnm, np);
	else
		snprintf(str, 200, "area_%d_%s_np%d_inter", Nmc, fnm, np);

	printf(" str %s \n", str);
	C = fopen(str, "w");

	dx = to/ (num-1);
	for ( i =1 ;i< num ;i ++){
		AA = calc_A(dx*i, mcavg, np);
		uAA = calc_uA_covmatrix(dx*i, mcavg, mccov, np);
		fprintf(C, " %g  %g  %g \n", dx*i, AA, uAA);
	}
	fclose(C);

	if ( !usecorr)
		snprintf(str, 200, "back_area_%d_%s_np%d_nocorr_inter", Nmc, fnm, np);
	else
		snprintf(str, 200, "back_area_%d_%s_np%d_inter", Nmc, fnm, np);

	printf(" str %s \n", str);
	C = fopen(str, "w");

	fprintf(C, "#hc, Er, u(Er;S), u(Er; Ah), u(Er; Apar), u(Er; cov), u(Er), A, u(A;h), u(A;par), u(A), Hit, u(Hit; F), u(Hit;Ah), u(Hit; Apar), u(Hit) \n");
	printf( "#hc, Er, u(Er;S), u(Er; Ah), u(Er; Apar), u(Er; cov), u(Er), A, u(A;h), u(A;par), u(A), Hit, u(Hit; F), u(Hit;Ah), u(Hit; Apar), u(Hit) \n");
	for ( i = 0 ; i < n; i ++){
		calc_unc_analysis(&data, &sdata, &fdata, i, mcavg, mccov, np, &Erdata, &Adata, &Hdata);
		fprintf(C, " %g  %g  %g  %g  %g  %g  %g  %g  %g  %g  %g %g %g %g %g %g \n", data.hc[i], 
				Erdata.Ercalc, Erdata.uErcalcS, Erdata.uErcalcAhc, 
				Erdata.uErcalcApar, Erdata.uErcalccov, Erdata.uErcalc,
				Adata.A, Adata.uAhc, Adata.uApar, Adata.uA,
				Hdata.H, Hdata.uF, Hdata.uHAhc, Hdata.uHApar, Hdata.uH);
	}
	fclose(C);




	free(beta);
	free(stdev_beta);
	free(cov);
	free_area_data(&data);
	free_s_data(&sdata);
	free_f_data(&fdata);
	free(x);
	free(ux);
	free(xnoise);
	free(mcbeta);
	free(mcstd);
	free(mcresvar);
	free(mcressum);
	free(mcavg);
	free(xhist);
	free(yhist);
	for (i =0; i< np; i++){
		free(mccov[i]);
	}
	free(mccov);

	return 0;
}
