#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <unistd.h>


typedef struct {
	int ndata;
	double *hc;
	double *Ac;
	double *uhc;
	double *uAc;
	double *rho;
} Data;	


int interpolate_function(double *y, double *x, double *uy, double *ux, int n, double xt, double *val, double *unc)
{

	int i, asc;
	double uval;
	double z;

    asc = (x[0] < x[n-1] ? 1 : 0);

    if (xt < x[0] || xt> x[n-1]){
	    printf(" xt %g out of range %g %g  \n", xt, x[0], x[n-1]);
        return 0;
    }

    i = 0;
    if (asc) {
        while (i < n-1 && x[i] < xt)
            i++;
    } else {
        while (i < n-1 && x[i] > xt)
            i++;
    }
    
    *val = (xt - x[i-1]) / (x[i] - x[i-1])*(y[i]-y[i-1]) + y[i-1];
    

    if (unc == NULL)
	    return 1;

    uval = 0;
    /* dyt / dy_i */
    z = (xt - x[i-1])/(x[i] - x[i-1]) * uy[i] ;
    uval += z*z;
    /* dyt / dy_i-1 */
    z = -(xt - x[i])/(x[i] - x[i-1]) * uy[i-1] ;
    uval += z*z;
    /* dyt / dx_i-1 */
    z = (xt - x[i])/(x[i] - x[i-1]) * (y[i] - y[i-1])/(x[i] - x[i-1]) * ux[i-1] ;
    uval += z*z;
    /* dyt / dx_i */
    z = -(xt - x[i-1])/(x[i] - x[i-1]) * (y[i] - y[i-1])/(x[i] - x[i-1]) * ux[i] ;
    uval += z*z;

    *unc = sqrt(uval);
    return 2;

}


void print_help()
{
	printf(" calculate area and uncertainty by interpolation.\n");
	printf(" -h    show this help\n");
	printf(" -f    input file \n");
}

int read_area_data( char *fnm, Data *data)
{
	FILE *file;
	int ndata = 0;
	int i;
	size_t size; 
	char *line = NULL;
	double x1, x2, x3, x4, x5, x6;

	file = fopen(fnm, "r");

	if (!file)
		return -1;
	
	while (getline(&line, &size, file) != EOF) {
		if (strstr(line, "#") == NULL)
				ndata ++;
	}
	printf("ndata %d \n", ndata);

	data->ndata = ndata;
	data->hc = (double *)malloc(ndata *sizeof(double));
	data->Ac = (double *)malloc(ndata *sizeof(double));
	data->uhc = (double *)malloc(ndata *sizeof(double));
	data->uAc = (double *)malloc(ndata *sizeof(double));
	data->rho = (double *)malloc(ndata *sizeof(double));

	rewind(file);

	ndata = 0; 
	while (getline(&line, &size, file) != EOF) {
		if (strstr(line, "#") == NULL){
			i = sscanf(line, "%lf %lf %lf %lf %lf %lf", &x1, &x2, &x3, &x4, &x5, &x6);
			if (i == 6 ){
				data->hc[ndata] = x1;
				data->Ac[ndata] = x2*1e6;
				data->uhc[ndata] = x3;
				data->uAc[ndata] = x4*1e6;
				data->rho[ndata] = x6;
				ndata++;
			}
		}
	}
	printf("ndata %d data->ndata %d \n", ndata, data->ndata);
	if (ndata != data -> ndata ){
		printf("read only %d data instead of %d \n", ndata, data->ndata);
		return ndata;
	}

	return ndata;
}
void free_area_data(Data *data){
	free(data->hc);
	free(data->Ac);
	free(data->uhc);
	free(data->uAc);
	free(data->rho);
}

int main (int argc, char *argv[])
{
	int c, i ;
	char *fnm = NULL;
	int isok;

	Data data;

	double from, to;
	int num = 200;
	double dx, x, AA, uAA;
	FILE *C;
	char str[200];



	if (argc == 1 ){
		print_help();
		exit(0);
	}

	while (( c = getopt (argc , argv, "f:h")) != -1 ){
		switch (c)
		{
			case 'f':
				fnm = optarg;
				break;
			case 'h':
			default:
				print_help();
				exit(0);
				break;
		}
	}

	if (!fnm){
		print_help();
		exit(0);
	}

	isok = read_area_data(fnm, &data);
	if (!isok){
		exit(2);
	}
	
	/*
	for (i = 0;i< data.ndata; i++)
		printf("%g %g \n", data.hc[i], data.Ac[i]);
		*/

	snprintf(str, 200, "area_%s_interpol", fnm);
	printf("Output to file %s \n", str);
	C = fopen(str, "w");
	from = data.hc[0];
	to = data.hc[data.ndata-1];
	dx = (to - from)/ (num-1);


	for ( i =0 ;i< num ;i ++){
		x = from + i* dx;
		if (interpolate_function(data.Ac, data.hc, data.uAc, data.uhc, data.ndata, x, &AA, &uAA) == 0)
			printf("Could not interpolate at %g \n", x);
		else
			fprintf(C, " %g  %g  %g \n", x, AA, uAA);
	} 
	fclose(C);



	free_area_data(&data);
	return 0;
}
