#include "instrument.h"

#include <math.h>
#include <stdlib.h>
#include <string.h>


#define NU_DEFAULT 0.3
#define NUI_DEFAULT 0.07
#define EI_DEFAULT 1141e9
#define UNC_NU_DEFAULT 0.1
#define UNC_NUI_DEFAULT 0.1
#define UNC_EI_DEFAULT 100e9
#define SIGMA_H_DEFAULT 0.1
#define SIGMA_F_DEFAULT 0.0001


static void fit_area_polynom(Area *area, gdouble *a, gdouble *b);
static gdouble interpolate_area_function(Area *area, gdouble h);
static gdouble interpolate_area_function_derivative(Area *area, gdouble h);
static gdouble polynom_eval_derivative(Area *area, gdouble h);

static void
fit_area_polynom(Area *area, gdouble *a, gdouble *b)
{
    gdouble sx4, sx3, sx2, syx2, syx;
    gint n;
    gint i;
    gdouble *x, *y;

    x = area->xdata->data;
    y = area->ydata->data;
    n = area->xdata->res;

    sx4 = 0;
    sx3 = 0;
    sx2 = 0;
    syx2 = 0;
    syx = 0;
    for (i = 0; i < n; i++) {
        sx4 += x[i]*x[i]*x[i]*x[i];
        sx3 += x[i]*x[i]*x[i];
        sx2 += x[i]*x[i];
        syx2 += y[i]*x[i]*x[i];
        syx += y[i]*x[i];
    }

    *a = (syx2*sx2 - syx*sx3) / (sx4*sx2 - sx3*sx3);
    *b = (syx*sx4 - syx2*sx3) / (sx4*sx2 - sx3*sx3);
}

static gdouble
interpolate_area_function(Area *area, gdouble h)
{
    gint i,n;
    gboolean asc;
    gdouble *x;
    gdouble val;
    gdouble rest, a, b;

    x = area->xdata->data;
    n = area->xdata->res;

    asc = (x[0] < x[n-1] ? TRUE : FALSE);

    if (h < 0)
        return 0;

    i = 0;
    if (asc) {
        while (i < n-1 && x[i] < h)
            i++;
    } else {
        while (i < n-1 && x[i] > h)
            i++;
    }
    
    if (h > x[n-1]) {
        fit_area_polynom(area, &a, &b);
        val = a*h*h + b*h;
    } else {
        rest = (h - x[i-1]) / (x[i] - x[i-1]);
        val = gwy_data_line_get_dval(area->ydata, i - 1 + rest + 0.5, GWY_INTERPOLATION_BSPLINE);
    }

    return val;
}

static gdouble
interpolate_area_function_derivative(Area *area, gdouble h)
{
    gint i, n;
    gboolean asc;
    gdouble *x;
    gdouble dval;
    GwyDataLine *darea;
    gdouble rest, a, b;

    x = area->xdata->data;
    n = area->xdata->res;

    darea = gwy_data_line_new_alike(area->ydata, FALSE);
    for (i = 0; i < darea->res; i++)
	    darea->data[i] = gwy_data_line_get_der(area->ydata, i);

    asc = (x[0] < x[n-1] ? TRUE : FALSE);

    if (h < 0)
        return 0;

    i = 0;
    if (asc) {
        while (i < n-1 && x[i] < h)
            i++;
    }
    else {
        while (i < n-1 && x[i] > h)
            i++;
    }
    
    if (h > x[n-1]) {
        fit_area_polynom(area, &a, &b);
        dval = a*2*h + b;
    } else {
        rest = (h - x[i-1]) / (x[i] - x[i-1]);
        dval = gwy_data_line_get_dval(darea, i - 1 + rest + 0.5, GWY_INTERPOLATION_BSPLINE);
    }

    g_object_unref(darea);

    return dval;
}

static gdouble
polynom_eval_derivative(Area *area, gdouble h) 
{
    gint i;
    gdouble val = 0;

    for (i = 0; i < area->npoly; i++) {
        val += area->polycoef[i] * area->polypower[i]*pow(h, area->polypower[i]-1);
    }

    return val;
}

void
init_Area(Area *area)
{
    gint i;

    area->filename = NULL;

    area->xdata = NULL;
    area->ydata = NULL;
    area->xmax = 0;

    area->mode = AREA_TRIVIAL;

    area->npoly = 6;

    area->polycoef = (gdouble*)g_malloc(area->npoly * sizeof(gdouble)); // neuvolni se nikdy, valgrind error, zatim nechat
    area->polycoef[0] = 24.5; 
    for (i = 1;  i < area->npoly; i++)
	    area->polycoef[i] = 0;

    area->polypower = (gdouble*)g_malloc(area->npoly * sizeof(gdouble)); // neuvolni se nikdy, valgrind error, zatim nechat
    for (i = 0;  i < area->npoly; i++)
	    area->polypower[i] = pow(2, 1-i);

    area->polylabel = (gchar**)g_malloc(area->npoly * sizeof(gchar*)); // neuvolni se nikdy, valgrind error, zatim nechat
    area->polylabel[0] = "2";
    area->polylabel[1] = "1";
    area->polylabel[2] = "1/2";
    area->polylabel[3] = "1/4";
    area->polylabel[4] = "1/8";
    area->polylabel[5] = "1/16";
    /* area->polylabel[6] = "1/32"; */

    area->coeff_filename = NULL;
}

void
init_Instdata(Instdata *instdata)
{
    instdata->Ei = EI_DEFAULT;
    instdata->nui = NUI_DEFAULT;
    instdata->nu = NU_DEFAULT;
    instdata->uEi = UNC_EI_DEFAULT;
    instdata->unui = UNC_NUI_DEFAULT;
    instdata->unu = UNC_NU_DEFAULT;
    instdata->sigma_h = SIGMA_H_DEFAULT;
    instdata->sigma_F = SIGMA_F_DEFAULT;
    instdata->delta = SIGMA_F_DEFAULT*SIGMA_F_DEFAULT/SIGMA_H_DEFAULT/SIGMA_H_DEFAULT;
}


/* not static as it is presently used in the area dialog */
gdouble
polynom_eval(Area *area, gdouble h) 
{
    gint i;
    gdouble val = 0;

    for (i = 0; i < area->npoly; i++)
        val += area->polycoef[i] * pow(h, area->polypower[i]);

    return val;
}

gdouble
eval_area(Area *area, gdouble h)
{
    gdouble Aphc;

    if (h < 0)
	    return 0;

    switch (area->mode) {
        case (AREA_TRIVIAL):
	    Aphc = 24.5 * h*h;
	    break;
        case (AREA_POLYNOM):
	    Aphc = polynom_eval(area, h);
	    break;
        case (AREA_DATA):
	    Aphc = interpolate_area_function(area, h);
	    break;
    }

    return Aphc;
}

gdouble
eval_area_derivative(Area *area, gdouble h)
{
    gdouble dAphc;

    if (h < 0)
	    return 0;

    switch (area->mode) {
        case (AREA_TRIVIAL):
	    dAphc = 24.5 * 2*h;
	    break;
        case (AREA_POLYNOM):
	    dAphc = polynom_eval_derivative(area, h);
	    break;
        case (AREA_DATA):
	    dAphc = interpolate_area_function_derivative(area, h);
	    break;
    }
    return dAphc;
}

/* zkonsolidovat s write_area_file */
gchar*
write_area_label(Area *area)
{
    gchar **strpoly;
    gdouble eps = 0.001;
    gint nsize = 64;
    gchar *str;
    gint i;

    if (area->mode == AREA_DATA) {
	    str = g_strdup(area->filename);
    } else {
	    strpoly = (gchar**)g_malloc((area->npoly + 1) * sizeof(gchar*));
	    for (i = 0; i < area->npoly; i++)
	        strpoly[i] = (gchar*)g_malloc(nsize * sizeof(gchar));

	    strpoly[area->npoly] = NULL;

	    for (i = 0; i < area->npoly; i++) {
	        if (fabs(area->polycoef[i]) > eps) {
	    	    if (i == 0)
	    	        g_snprintf(strpoly[i], nsize, "%.3gx<sup>%s</sup>", area->polycoef[i], area->polylabel[i]);
	    	    else if (area->polycoef[i] > eps)
	    	        g_snprintf(strpoly[i], nsize, " + %.3gx<sup>%s</sup>", area->polycoef[i], area->polylabel[i]);
	    	    else 
	    	        g_snprintf(strpoly[i], nsize, " - %.3gx<sup>%s</sup>", fabs(area->polycoef[i]), area->polylabel[i]);
	        } else
	    	    strpoly[i] = g_strdup("");
	    }
	    str = g_strjoinv("", strpoly);
	    g_strfreev(strpoly);
    }

    return str;
}

/* zkonsolidovat s write_area_label */
gchar*
write_area_file(Area *area)
{
    gchar **strpoly;
    gdouble eps = 0.001;
    gint nsize = 64;
    gchar *str;
    gint i;
    gchar buffer[1000];

    if (area->mode == AREA_DATA) {
	    g_snprintf(buffer, sizeof(buffer), "#Raw data file: %s \n", area->filename);
	    str = g_strdup(buffer);
    } else {
	    strpoly = (gchar**)g_malloc((area->npoly + 1) * sizeof(gchar*));
	    for (i = 0; i < area->npoly; i++)
	        strpoly[i] = (gchar*)g_malloc(nsize * sizeof(gchar));

	    strpoly[area->npoly] = NULL;

	    for (i = 0; i < area->npoly; i++) {
	        if (fabs(area->polycoef[i]) > eps) {
	    	    if (i == 0)
	    	        g_snprintf(strpoly[i], nsize, "%gx^%s", area->polycoef[i], area->polylabel[i]);
	    	    else if (area->polycoef[i] > eps)
	    	        g_snprintf(strpoly[i], nsize, " + %gx^%s", area->polycoef[i], area->polylabel[i]);
	    	    else 
	    	        g_snprintf(strpoly[i], nsize, " - %gx^%s", fabs(area->polycoef[i]), area->polylabel[i]);
	        } else
	    	    strpoly[i] = g_strdup("");
	    }
	    str = g_strjoinv("", strpoly);
	    g_strfreev(strpoly);
	    g_snprintf(buffer, sizeof(buffer), "#Polynomial area function:  %s \n", str);
	    str = g_strdup(buffer);
    }

    return str;
}


