#ifndef NOFORTRAN

#include "tool_hertz_odr_mc.h"
#include "tool_hertz_odr.h"

#include "controls.h"
#include "datatypes.h"
#include "fit-utils.h"
#include "gui-utils.h"
#include "indent-toolbox-common.h"
#include "mc-utils.h"

#include <math.h>

#include <gtk/gtk.h>

#include <app/file.h>

#define N_FIX 1.5

#define NPARAM 3  // Er, Eit or Radius
#define MAXBAD 5 //maximum percentage of nonsense data

static const gchar *logfnm = "fit.log.hz.mc";

void
init_HertzODRMCdata (HertzODRMCdata *hzodrmc, gint Nmc)
{
	gint i;

	//number of iterations
	hzodrmc->Nmc = Nmc;
	printf("initialize for %d iterations \n", hzodrmc->Nmc);

	//how much had to be skipped 
	hzodrmc->skipmc = 0;

	//dynamically allocate
	hzodrmc->hertzodrmc = (HertzODRdata *)g_malloc(hzodrmc->Nmc*sizeof(HertzODRdata));
	hzodrmc->mcavg = (gdouble *)g_malloc(NPARAM*sizeof(gdouble));
	hzodrmc->mcstd = (gdouble *)g_malloc(NPARAM*sizeof(gdouble));
	hzodrmc->mcdd = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));
	hzodrmc->mcxhist = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));
	hzodrmc->mcyhist = (gdouble **) g_malloc(NPARAM*sizeof(gdouble *));

	hzodrmc->mcnstat = (gint)floor(3.49*cbrt(hzodrmc->Nmc)+0.5);
	for (i = 0; i< NPARAM; i++){
		hzodrmc->mcdd[i] = (gdouble *) g_malloc(hzodrmc->Nmc*sizeof(gdouble));
		hzodrmc->mcxhist[i] = (gdouble *) g_malloc(hzodrmc->mcnstat*sizeof(gdouble));
		hzodrmc->mcyhist[i] = (gdouble *) g_malloc(hzodrmc->mcnstat*sizeof(gdouble));
	}
}

void
destroy_HertzODRMCdata (HertzODRMCdata *hzodrmc)
{
	gint i;


	//free everything
	g_free(hzodrmc->mcavg);
	g_free(hzodrmc->mcstd);

	for (i = 0; i< NPARAM; i++){
		g_free(hzodrmc->mcdd[i]);
		g_free(hzodrmc->mcxhist[i]);
		g_free(hzodrmc->mcyhist[i]);
	}

	g_free(hzodrmc->mcdd);
	g_free(hzodrmc->mcxhist);
	g_free(hzodrmc->mcyhist);

	hzodrmc->mcnstat = 0;
	hzodrmc->skipmc = 0;
	
	for (i = 0; i < hzodrmc->Nmc; i++){
		g_free(hzodrmc->hertzodrmc[i].logfnm);
	}

	g_free(hzodrmc->hertzodrmc);
}

static gboolean 
nparam_to_string(gint i, gchar *str, gint nstr)
{

	switch (i){
		case 0:
			g_snprintf(str,nstr,"E_r/GPa");
			break;
		case 1:
			g_snprintf(str,nstr,"E_IT/GPa");
			break;
		case 2:
			g_snprintf(str,nstr,"radius/nm");
			break;
		default:
			printf("Unhandled number of results.\n");
			return FALSE;
	}
	return TRUE;
}


gboolean hertz_odr_mc_write_file(gchar *filename, Args *args, FileSaveStatus *filestatus) 
{
	FILE *file;
	HertzODRdata *hzodr;
	gint i,j ;
	gchar str[200];
	gboolean is;

	if (args->fddata.hload->res == 0 || args->fddata.Fload->res==0) {
		*filestatus = SAVE_FILE_NO_DATA;
		return FALSE;
	}

	file = fopen(filename, "w");
	if (!file) {
		*filestatus = SAVE_FILE_COULD_NOT_OPEN;
		return FALSE;
	}

	fprintf(file, "# %s \n", args->fddata.filename);
	fprintf(file, "#Monte Carlo simulation of HertzODR analysis\n");
	fprintf(file, "#Input uncertainties \n");
	fprintf(file, "#u(h) = %g nm \n", args->hertzodrunc.uh);
	fprintf(file, "#u(F) = %g mN \n", args->hertzodrunc.uF);
	fprintf(file, "#Number of iterations  = %d  \n", args->hertzodrmc.Nmc);
	fprintf(file, "\n");

	fprintf(file, "#Parameters of HertzODR analysis \n");

	switch(args->hertzodrdata.mode){
		case R_MODE:
			fprintf(file, "#tip radius: %g nm\n", args->hertzodrdata.radius*1e9);
			fprintf(file, "#nu: %g                 \n", args->instdata.nu);
			fprintf(file, "#nu_indenter: %g        \n", args->instdata.nui);
			fprintf(file, "#E_indenter: %g      GPa\n", args->instdata.Ei * 1e-9); //TODO units
			fprintf(file, "#fitted range:  [%g, %g ] nm \n", args->hertzodrdata.range[0], args->hertzodrdata.range[1]);
			break;
		case ER_MODE:

			fprintf(file, "#Er: %g      GPa\n", args->hertzodrdata.Er ); //TODO units
			fprintf(file, "#fitted range:  [%g, %g ] nm \n", args->hertzodrdata.range[0], args->hertzodrdata.range[1]);
			break;
		case EIT_MODE:
			fprintf(file, "#Eit: %g      GPa\n", args->hertzodrdata.Eit ); //TODO units
			fprintf(file, "#nu: %g                 \n", args->instdata.nu);
			fprintf(file, "#nu_indenter: %g        \n", args->instdata.nui);
			fprintf(file, "#E_indenter: %g      GPa\n", args->instdata.Ei * 1e-9); //TODO units
			fprintf(file, "#fitted range:  [%g, %g ] nm \n", args->hertzodrdata.range[0], args->hertzodrdata.range[1]);
	}
	if (args->hertzodrdata.ifixb[2] == 0)
			fprintf(file, "#n: %g fixed \n", N_FIX);

	fprintf(file, "#");
	if (args->hertzodrmc.skipmc)
		fprintf(file, "Fit failed in %d cases, the corresponding data area NOT included in the  estimates of mean and uncertainty but ARE included in the histograms.", args->hertzodrmc.skipmc );
	fprintf(file, "\n");

	fprintf(file, "#\n\n");


	fprintf(file, "#Results \n");

	switch(args->hertzodrdata.mode){
		case R_MODE:
			for (i = 0; i < 2; i++){
				is = nparam_to_string(i, str, sizeof(str));
				fprintf(file, "#%s = (%g ± %g ) \n", str, args->hertzodrmc.mcavg[i], args->hertzodrmc.mcstd[i]);
			}
			break;
		case ER_MODE:
		case EIT_MODE:
			is = nparam_to_string(2, str, sizeof(str));
			fprintf(file, "#%s = (%g ± %g ) \n", str, args->hertzodrmc.mcavg[2], args->hertzodrmc.mcstd[2]);
			break;
	}

	fprintf(file, "\n\n\n");


	// Histograms

	if (args->hertzodrmc.mcnstat >0){
		fprintf(file,"#Histogram\n");
		switch(args->hertzodrdata.mode){
			case R_MODE:
				for (i=0; i < 2; i++){
					fprintf(file,"#Histogram\n");
					is = nparam_to_string(i, str, sizeof(str));
					if (is){
						fprintf(file,"#%s    count/a.u.  \n", str);
						for (j = 0;j < args->hertzodrmc.mcnstat; j++)
							fprintf(file, "%g %g \n", args->hertzodrmc.mcxhist[i][j], args->hertzodrmc.mcyhist[i][j]);
						fprintf(file, "\n \n");
					}
				}
				break;
			case ER_MODE:
			case EIT_MODE:
				i=2;
				fprintf(file,"#Histogram\n");
				is = nparam_to_string(i, str, sizeof(str));
				if (is){
					fprintf(file,"#%s    count/a.u.  \n", str);
					for (j = 0;j < args->hertzodrmc.mcnstat; j++)
						fprintf(file, "%g %g \n", args->hertzodrmc.mcxhist[i][j], args->hertzodrmc.mcyhist[i][j]);
					fprintf(file, "\n \n");
				}
		}
	}


	fprintf(file, "\n\n\n");

	//Full data
	fprintf(file, "#Full data \n");
	fprintf(file, "#");
	switch(args->hertzodrdata.mode){
		case R_MODE:
			for ( j = 0; j< 2; j++){
				is = nparam_to_string(j, str, sizeof(str));
				fprintf(file, "%s ", str);
			}
			break;
		case ER_MODE:
		case EIT_MODE:
			is = nparam_to_string(2, str, sizeof(str));
			fprintf(file, "%s ", str);
			break;
	}
	fprintf(file,"\n");

	hzodr= args->hertzodrmc.hertzodrmc;

	if (args->hertzodrdata.mode == R_MODE){
		for (i = 0; i< args->hertzodrmc.Nmc; i++){
			fprintf(file, MODUL, hzodr->Er);
			fprintf(file,"  ");
			fprintf(file, MODUL, hzodr->Eit);
			fprintf(file,"  \n");
			hzodr++;
		}
	}
	else {
		for (i = 0; i< args->hertzodrmc.Nmc; i++){
			fprintf(file, DEPTH, hzodr->radius*1e9);
			fprintf(file,"  \n");
			hzodr++;
		}
	}
	fclose(file); 
	*filestatus = SAVE_FILE_OK;
	return TRUE;
}


static void hertz_odr_mc_hist_radius( Data *data)
{
	HertzODRMCControls *ctrls;
	GtkWidget *graph;
	GwyGraphModel *graph_model;
	GwyGraphCurveModel *cmodel;
	gint j=2;

	ctrls = data->controls->hertzodrmccontrols;

	printf("show radius histogram \n");
	if (GTK_IS_WIDGET(ctrls->hist_window[j]))
	{
		gtk_window_present(GTK_WINDOW ( ctrls->hist_window[j]));
		return;
	}

	ctrls->hist_window[j] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->hist_window[j]), "Probability distribution of R");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->hist_window[j]), 300, 200); 
	g_signal_connect (ctrls->hist_window[j], "delete_event", G_CALLBACK (gtk_widget_destroy),NULL); // CHECK

	graph_model = gwy_graph_model_new();
	g_object_set(graph_model, "title", "PDF R", NULL);
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_BOTTOM, "R / nm");
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_LEFT, "count / a.u.");	

	graph = gwy_graph_new(graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(graph), 200, 150);

	gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(graph), GWY_GRAPH_STATUS_XSEL);



	cmodel = gwy_graph_curve_model_new();

	g_object_set(cmodel,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "PDF(R)",
			"point-size", 3,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
			NULL);
	gwy_graph_model_add_curve(graph_model, cmodel);

	gwy_graph_curve_model_set_data(cmodel,data->args->hertzodrmc.mcxhist[j], data->args->hertzodrmc.mcyhist[j], data->args->hertzodrmc.mcnstat);

	gtk_container_add(GTK_CONTAINER(ctrls->hist_window[j]), graph);
	gtk_widget_show_all(ctrls->hist_window[j]);
}


static void hertz_odr_mc_hist_Er( Data *data)
{
	HertzODRMCControls *ctrls;
	GtkWidget *graph;
	GwyGraphModel *graph_model;
	GwyGraphCurveModel *cmodel;
	gint j=0;

	ctrls = data->controls->hertzodrmccontrols;

	printf("show Er histogram \n");
	if (GTK_IS_WIDGET(ctrls->hist_window[j]))
	{
		gtk_window_present(GTK_WINDOW ( ctrls->hist_window[j]));
		return;
	}

	ctrls->hist_window[j] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->hist_window[j]), "Probability distribution of E_r");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->hist_window[j]), 300, 200); 
	g_signal_connect (ctrls->hist_window[j], "delete_event", G_CALLBACK (gtk_widget_destroy),NULL); // CHECK

	graph_model = gwy_graph_model_new();
	g_object_set(graph_model, "title", "PDF E_r", NULL);
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_BOTTOM, "E_r / GPa");
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_LEFT, "count / a.u.");	

	graph = gwy_graph_new(graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(graph), 200, 150);

	gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(graph), GWY_GRAPH_STATUS_XSEL);



	cmodel = gwy_graph_curve_model_new();

	g_object_set(cmodel,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "PDF(E_r)",
			"point-size", 3,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
			NULL);
	gwy_graph_model_add_curve(graph_model, cmodel);

	gwy_graph_curve_model_set_data(cmodel,data->args->hertzodrmc.mcxhist[j], data->args->hertzodrmc.mcyhist[j], data->args->hertzodrmc.mcnstat);

	gtk_container_add(GTK_CONTAINER(ctrls->hist_window[j]), graph);
	gtk_widget_show_all(ctrls->hist_window[j]);
}


static void hertz_odr_mc_hist_Eit( Data *data)
{
	HertzODRMCControls *ctrls;
	GtkWidget *graph;
	GwyGraphModel *graph_model;
	GwyGraphCurveModel *cmodel;
	gint j=1;

	ctrls = data->controls->hertzodrmccontrols;

	if (GTK_IS_WIDGET(ctrls->hist_window[j]))
	{
		gtk_window_present(GTK_WINDOW ( ctrls->hist_window[j]));
		return;
	}

	ctrls->hist_window[j] = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->hist_window[j]), "Probability distribution of E_IT");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->hist_window[j]), 300, 200); 
	g_signal_connect (ctrls->hist_window[j], "delete_event", G_CALLBACK (gtk_widget_destroy),NULL); // CHECK

	graph_model = gwy_graph_model_new();
	g_object_set(graph_model, "title", "PDF E_IT", NULL);
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_BOTTOM, "E_IT / GPa");
	gwy_graph_model_set_axis_label (graph_model, GTK_POS_LEFT, "count / a.u.");	

	graph = gwy_graph_new(graph_model);
	gtk_widget_set_size_request(GTK_WIDGET(graph), 200, 150);

	gwy_graph_enable_user_input(GWY_GRAPH(graph), FALSE);
	gwy_graph_set_status(GWY_GRAPH(graph), GWY_GRAPH_STATUS_XSEL);



	cmodel = gwy_graph_curve_model_new();

	g_object_set(cmodel,
			"mode", GWY_GRAPH_CURVE_LINE_POINTS,
			"description", "PDF(E_IT)",
			"point-size", 3,
			"point-type", GWY_GRAPH_POINT_CIRCLE,
			"color", CURVE_COLOR_UNLOAD,
			"line-style", GDK_LINE_SOLID,
			NULL);
	gwy_graph_model_add_curve(graph_model, cmodel);

	gwy_graph_curve_model_set_data(cmodel,data->args->hertzodrmc.mcxhist[j], data->args->hertzodrmc.mcyhist[j], data->args->hertzodrmc.mcnstat);

	gtk_container_add(GTK_CONTAINER(ctrls->hist_window[j]), graph);
	gtk_widget_show_all(ctrls->hist_window[j]);

}


static void hertz_odr_mc_save_data( Args *args)
{
	GtkWidget *dialog;
	gchar *filename;
	FileSaveStatus filestatus;
	gboolean file_ok;
	gint response;

	dialog = gtk_file_chooser_dialog_new("Save File",
			NULL,
			GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_SAVE, GTK_RESPONSE_OK,
			NULL);

	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), gwy_app_get_current_directory());

	file_ok = FALSE;
	response = gtk_dialog_run (GTK_DIALOG ( dialog));
	while (  response == GTK_RESPONSE_OK && !file_ok)
	{
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
		if (gwy_app_file_confirm_overwrite(dialog)){
			hertz_odr_mc_write_file(filename, args, &filestatus);
			file_ok = (filestatus == SAVE_FILE_OK);
		}
		g_free(filename);
		if (!file_ok){
			show_save_error(filestatus, filename);
			response = gtk_dialog_run (GTK_DIALOG ( dialog));
		}
	}
	gtk_widget_destroy ( dialog);
}

void 
hertz_odr_mc_close_window(Data *data)
{

	gint j;

	if (data->controls->hertzodrmccontrols->has_window){
		gtk_widget_destroy(data->controls->hertzodrmccontrols->window);

		destroy_HertzODRMCdata(&(data->args->hertzodrmc));
		data->controls->hertzodrmccontrols->has_window = FALSE;
		for (j=0; j< NPARAM; j++){
			if (GTK_IS_WIDGET(data->controls->hertzodrmccontrols->hist_window[j])){
				gtk_widget_destroy(data->controls->hertzodrmccontrols->hist_window[j]);
			}
		}
		g_free(data->controls->hertzodrmccontrols->hist_window);
		g_free(data->controls->hertzodrmccontrols->button_hist);
	}
}

void hertz_odr_uncertainty_montecarlo_run_calc( Args *args)
{
	gint istart, iend, ndata;
	gint i, j, Nmc;
	HertzODRdata *hzodr;
	HertzODRMCdata *mc;
	GwyDataLine *h, *F;
	GwyDataLine *hnew, *Fnew;
	GwyDataLine *hnoise, *Fnoise;
	GRand *rng;
	gint nn, nremove;
	gdouble **dd;
	gdouble *xhist, *yhist;
	gboolean hist;
	gdouble estimate[3];

	//not necessary to check range, couldn't get here in the first place

	//find appropriate data, which should be fitted, must use real fitted range
	range_to_indices(args->hertzodrdata.range[0], args->hertzodrdata.range[1], args->fddata.hload, FALSE, &istart, &iend, &ndata);
	// if there are not enough data in the selected range, not fitting can be performed
	ndata = MIN(ndata, args->fddata.hload->res - istart);

	if (ndata < 2){
		return;
	}

	//use results of previous fit as input estimates
	estimate[0]= args->hertzodrdata.gamma;
	estimate[1]= args->hertzodrdata.h0;
	estimate[2]= args->hertzodrdata.n;


	//get data to be fitted
	h = gwy_data_line_part_extract(args->fddata.hload, istart, ndata);
	F = gwy_data_line_part_extract(args->fddata.Fload, istart, ndata);


	Nmc = args->hertzodrmc.Nmc;
	hzodr= args->hertzodrmc.hertzodrmc;
	//prepare data
	for (i=0; i< Nmc ;i++){
		init_HertzODRdata(hzodr);
		g_free(hzodr->logfnm);
		hzodr->logfnm = create_log_filename_path(logfnm);
		hzodr++;
	}

	printf("copy params \n");
	hzodr= args->hertzodrmc.hertzodrmc;
	//copy input parameters
	for (i=0; i< Nmc ;i++){
		hzodr->mode = args->hertzodrdata.mode;
		for (j = 0; j< 3; j++)
			hzodr->ifixb[j] = args->hertzodrdata.ifixb[j];
		
		switch (hzodr->mode){
			case R_MODE:
				hzodr->radius = args->hertzodrdata.radius;
				break;
			case ER_MODE:
				hzodr->Er = args->hertzodrdata.Er;
				break;
			case EIT_MODE:
				hzodr->Eit = args->hertzodrdata.Eit;
				hzodr->Er = 1./((1-args->instdata.nui*args->instdata.nui)/(args->instdata.Ei*1e-9) + (1-args->instdata.nu*args->instdata.nu)/hzodr->Eit);
				break;
		}
		hzodr++;
	}

	// initialize random generator and datafield
	rng= g_rand_new();

	//prepare datafield for noise
	hnoise=gwy_data_line_new_alike(h,FALSE);
	Fnoise=gwy_data_line_new_alike(h,FALSE);

	//prepare datafield for varied fields
	hnew=gwy_data_line_new_alike(h,FALSE);
	Fnew=gwy_data_line_new_alike(F,FALSE);

	hzodr= args->hertzodrmc.hertzodrmc;
	for (i=0; i< Nmc; i++){

		//create noise for h-F data
		generate_uncorrelated_noise(hnoise, args->hertzodrunc.uh, rng);
		generate_uncorrelated_noise(Fnoise, args->hertzodrunc.uF, rng);

		//add noise to original h-F data
		if (!sum_data_lines(hnew,h, hnoise)){
			printf("Should not get here.\n");
			return;
		}
		if (!sum_data_lines(Fnew,F, Fnoise)){
			printf("Should not get here.\n");
			return;
		}

		//remove negative depths from data
		nremove = filter_negative_data(hnew, Fnew);

		//run fit
		hertz_odr_fit(hnew, Fnew, hzodr, &(args->fddata), &(args->instdata), estimate);

		if (nremove){
			gwy_data_line_resample(hnew, h->res, GWY_INTERPOLATION_NONE);
			gwy_data_line_resample(Fnew, F->res, GWY_INTERPOLATION_NONE);
		}


		hzodr++;
	}

	printf("run Monte Carlo \n");	
	g_object_unref(h);
	g_object_unref(F);
	g_object_unref(hnew);
	g_object_unref(Fnew);
	g_object_unref(hnoise);
	g_object_unref(Fnoise);
	g_rand_free(rng);


	//calculate averages and stdevs
	// Er, Eit
	
	mc = &(args->hertzodrmc);
	for (i = 0; i< NPARAM; i++){
		mc->mcavg[i] =0;
		mc->mcstd[i] =0;
	}

	hzodr= args->hertzodrmc.hertzodrmc;
	nn = 0;
	for (i = 0; i< Nmc; i++){
		//sanity check
		if (hzodr->fitinfo < 5)
		{
			mc->mcavg[0]+=hzodr->Er;
			mc->mcavg[1]+=hzodr->Eit;
			mc->mcavg[2]+=hzodr->radius*1e9; //TODO UNITS
			mc->mcstd[0]+=hzodr->Er*hzodr->Er;
			mc->mcstd[1]+=hzodr->Eit*hzodr->Eit;
			mc->mcstd[2]+=hzodr->radius*1e9*hzodr->radius*1e9; //TODO UNITS
			nn++;
		}
		hzodr++;
	}

	//use only reasonable data for average and standard deviation
	for (i = 0; i< NPARAM; i++){
		mc->mcavg[i] /= nn;
		mc->mcstd[i] /= nn;
		mc->mcstd[i]-= mc->mcavg[i]*mc->mcavg[i];
		mc->mcstd[i] = (mc->mcstd[i] < 0) ? 0 : mc->mcstd[i];
		mc->mcstd[i] = sqrt(mc->mcstd[i]);
	}

	// how many datasets were skipped
	args->hertzodrmc.skipmc = Nmc-nn;

	// Histograms
	//create data structure

	dd = args->hertzodrmc.mcdd;
	hzodr = args->hertzodrmc.hertzodrmc;
	for (i = 0; i< Nmc; i++){
		dd[0][i] = hzodr->Er;
		dd[1][i] = hzodr->Eit;
		dd[2][i] = hzodr->radius*1e9;
		hzodr++;
	}

	//prepare histogram structure

	for (i=0; i < NPARAM; i++){
		xhist = mc->mcxhist[i];
		yhist = mc->mcyhist[i];
		hist = create_histogram(dd[i], Nmc, xhist, yhist, mc->mcnstat );
		if (!hist){
			printf("Warning: No histogram could be created.\n");
			mc->mcnstat = -1;
			break;
		}
	}

}

void hertz_odr_uncertainty_montecarlo( Data *data)
{
	gint i, j;
	GtkWidget *label;
	GdkColor color;

	HertzODRMCControls *ctrls;
	HertzODRMCdata *hzodrmc;

	gdk_color_parse ("red", &color);

	//if there was a previous window, close it and any histograms, free data
	hertz_odr_mc_close_window(data);

	hzodrmc = &(data->args->hertzodrmc);

	// initialize for number of iterations which was chosen in the Uncertainties dialog
	init_HertzODRMCdata(hzodrmc, data->args->hertzodrunc.Nmc); 

	// run calculation
	hertz_odr_uncertainty_montecarlo_run_calc(data->args);

	//create window and display results
	ctrls = data ->controls->hertzodrmccontrols;

	ctrls->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(ctrls->window), "Monte Carlo calculation of uncertainties in HertzODR analysis");
	gtk_window_set_default_size(GTK_WINDOW(ctrls->window), 400, 300); 
	g_signal_connect_swapped (ctrls->window, "delete_event", G_CALLBACK (hertz_odr_mc_close_window), data);
	gtk_window_set_transient_for (GTK_WINDOW(ctrls->window), GTK_WINDOW(data->controls->maincontrols->window_main));
    gtk_window_set_destroy_with_parent (GTK_WINDOW(ctrls->window), TRUE);

	ctrls->frame_input = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_input),
			GTK_WIDGET (gwy_label_new_header("Input for Monte Carlo simulation")));

	ctrls->table_input = gtk_table_new(3, 4, FALSE);

	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_input), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_input), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_input), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (ctrls->frame_input), ctrls->table_input);

	i = 0;

	/*          u(h)             */

	gtk_table_attach(GTK_TABLE(ctrls->table_input), gwy_label_new_header("u(h)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);

	label = gtk_label_new(NULL);
	label_set_gdouble_format(label, data->args->hertzodrunc.uh, "%g");
	gtk_table_attach (GTK_TABLE (ctrls->table_input), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	gtk_table_attach(GTK_TABLE(ctrls->table_input), label_new_with_markup_left("nm"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);

	gtk_table_attach(GTK_TABLE(ctrls->table_input), label_new_with_markup_left("normal distribution"), 3, 4, i, i+1, GTK_FILL, 0, 0, 0);

	i++;

	/*          u(F)             */

	gtk_table_attach(GTK_TABLE(ctrls->table_input), gwy_label_new_header("u(F)"), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);

	label = gtk_label_new(NULL);
	label_set_gdouble_format(label, data->args->hertzodrunc.uF, "%g");
	gtk_table_attach (GTK_TABLE (ctrls->table_input), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	gtk_table_attach(GTK_TABLE(ctrls->table_input), label_new_with_markup_left("mN"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);

	gtk_table_attach(GTK_TABLE(ctrls->table_input), label_new_with_markup_left("normal distribution"), 3, 4, i, i+1, GTK_FILL, 0, 0, 0);

	i++;

	/*          Number of iteration */

	gtk_table_attach(GTK_TABLE(ctrls->table_input), gwy_label_new_header("Number of iterations "), 0, 1, i, i+1, GTK_FILL, 0, 0, 0);

	label = gtk_label_new(NULL);
	label_set_gint_format(label, hzodrmc->Nmc, "%d");
	gtk_table_attach (GTK_TABLE (ctrls->table_input), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

	// Results
	ctrls->frame_results = gtk_frame_new(NULL);
	gtk_frame_set_label_widget (GTK_FRAME (ctrls->frame_results),
			GTK_WIDGET (gwy_label_new_header("Results of Monte Carlo simulation")));

	ctrls->table_results = gtk_table_new(NPARAM+1, 4, FALSE);

	gtk_container_set_border_width (GTK_CONTAINER (ctrls->table_results), CTRLS_BORDER);
	gtk_table_set_col_spacings (GTK_TABLE (ctrls->table_results), CTRLS_TABLE_COL_SPACING);
	gtk_table_set_row_spacings (GTK_TABLE (ctrls->table_results), CTRLS_TABLE_ROW_SPACING);
	gtk_container_add ( GTK_CONTAINER (ctrls->frame_results), ctrls->table_results);

	i = 0;

	gtk_table_attach(GTK_TABLE(ctrls->table_results), gwy_label_new_header("mean"), 1, 2, i, i+1, GTK_FILL, 0, 0, 0);
	gtk_table_attach(GTK_TABLE(ctrls->table_results), gwy_label_new_header("stdev"), 2, 3, i, i+1, GTK_FILL, 0, 0, 0);

	i++;

	ctrls->button_hist = (GtkWidget **)g_malloc(NPARAM*sizeof(GtkWidget *));
	ctrls->hist_window = (GtkWidget **)g_malloc(NPARAM*sizeof(GtkWidget *));
	for (j = 0; j< NPARAM; j++)
		ctrls->hist_window[j] = NULL;


	if (data->args->hertzodrdata.mode == R_MODE){
		/*                       E_r           */

		gtk_table_attach(GTK_TABLE(ctrls->table_results),
				gwy_label_new_header("E<sub>r</sub>/GPa"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units

		label = gtk_label_new(NULL);
		label_set_gdouble_format(label, hzodrmc->mcavg[i-1], MODUL);
		gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

		label = gtk_label_new(NULL);
		label_set_gdouble_format(label, hzodrmc->mcstd[i-1], MODUL);
		gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

		ctrls->button_hist[i-1] = gtk_button_new();
		gtk_button_set_image (GTK_BUTTON (ctrls->button_hist[i-1]), gtk_image_new_from_stock (GWY_STOCK_GRAPH, GTK_ICON_SIZE_BUTTON));
		g_signal_connect_swapped (ctrls->button_hist[i-1], "clicked", G_CALLBACK (hertz_odr_mc_hist_Er), data);
		gtk_table_attach (GTK_TABLE (ctrls->table_results), ctrls->button_hist[i-1], 3, 4, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
		i++;

		/*                       E_IT          */

		gtk_table_attach(GTK_TABLE(ctrls->table_results),
				gwy_label_new_header("E<sub>it</sub>/GPa"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units

		label = gtk_label_new(NULL);
		label_set_gdouble_format(label, hzodrmc->mcavg[i-1], MODUL);
		gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

		label = gtk_label_new(NULL);
		label_set_gdouble_format(label, hzodrmc->mcstd[i-1], MODUL);
		gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

		ctrls->button_hist[i-1] = gtk_button_new();
		gtk_button_set_image (GTK_BUTTON (ctrls->button_hist[i-1]), gtk_image_new_from_stock (GWY_STOCK_GRAPH, GTK_ICON_SIZE_BUTTON));
		g_signal_connect_swapped (ctrls->button_hist[i-1], "clicked", G_CALLBACK (hertz_odr_mc_hist_Eit), data);
		gtk_table_attach (GTK_TABLE (ctrls->table_results), ctrls->button_hist[i-1], 3, 4, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
		i++;
	}
	else {
		/*                       radius                */

		i = 3;
		gtk_table_attach(GTK_TABLE(ctrls->table_results),
				gwy_label_new_header("radius/nm"), 0, 1, i, i+1, GTK_EXPAND | GTK_FILL, 0, 0, 0); //TODO units

		label = gtk_label_new(NULL);
		label_set_gdouble_format(label, hzodrmc->mcavg[i-1], DEPTH);
		gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

		label = gtk_label_new(NULL);
		label_set_gdouble_format(label, hzodrmc->mcstd[i-1], DEPTH);
		gtk_table_attach (GTK_TABLE (ctrls->table_results), label, 2, 3, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 

		ctrls->button_hist[i-1] = gtk_button_new();
		gtk_button_set_image (GTK_BUTTON (ctrls->button_hist[i-1]), gtk_image_new_from_stock (GWY_STOCK_GRAPH, GTK_ICON_SIZE_BUTTON));
		g_signal_connect_swapped (ctrls->button_hist[i-1], "clicked", G_CALLBACK (hertz_odr_mc_hist_radius), data);
		gtk_table_attach (GTK_TABLE (ctrls->table_results), ctrls->button_hist[i-1], 3, 4, i, i+1, GTK_FILL, GTK_FILL, 0, 0); 
		i++;
	}

	if (100.*data->args->hertzodrmc.skipmc/hzodrmc->Nmc > MAXBAD){
		label = gtk_label_new(NULL);
		label_set_gdouble_format(label, 100. * data->args->hertzodrmc.skipmc  /hzodrmc->Nmc,
					 "Warning: Approx %.1f %% of the results had to be removed!");
		gtk_widget_modify_fg(label, GTK_STATE_NORMAL, &color);
		gtk_table_attach(GTK_TABLE(ctrls->table_results), label, 0,1, i, i+1, GTK_FILL, GTK_FILL,0,0);
		i++;
	}
	// save data
	ctrls->button_save = gtk_button_new_from_stock (GTK_STOCK_SAVE);
	g_signal_connect_swapped (ctrls->button_save, "clicked", G_CALLBACK (hertz_odr_mc_save_data), data->args);

	ctrls->vbox = gtk_vbox_new(FALSE, CTRLS_SPACING);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_input, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->frame_results, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(ctrls->vbox), ctrls->button_save, FALSE, FALSE, 0);

	gtk_container_add(GTK_CONTAINER(ctrls->window), ctrls->vbox);
	gtk_widget_show_all(ctrls->window);
	ctrls->has_window = TRUE;

}

#endif
